// *** WARNING *** All data used by the 'rewritable' (aka 'user') code must be separated
// into a separate segment using the USERDATA macro. This prevents new variables in the
// user code from relocating variables used by the code which is NOT rewritten.
// *** ALSO NOTE *** USERDATA variables are never initialised by the C startup code, so
// a declaration of the form "USERDATA int fred = 5;" does not have the expected effect -
// the value of fred at startup is undefined, and the "=5" part is silently ignored. USERDATA
// variable must be explicitly initialised before use.

#define I2C_SDA_LO I2C_DDR = (I2C_DDR | SDA) & ~SCL
#define I2C_SCL_LO I2C_DDR = (I2C_DDR | SCL) & ~SDA
#define I2C_ALL_LO I2C_DDR = I2C_DDR | SDA | SCL
#define I2C_ALL_HI I2C_DDR = I2C_DDR & (~SDA & ~SCL)

USERDATA char I2CErrors;
USERDATA int I2C_MINTIME;

void I2CDelay(volatile int i)  REWRITABLE;
void I2CDelay(volatile int i)
{						// software delay loop
while (i--);
}

void I2CSendStart(void)  REWRITABLE;
void I2CSendStart(void)
{
I2CErrors = 0;											// reset error counter												// switch ports to output direction
I2C_ALL_HI;  					I2CDelay(I2C_MINTIME);	// all lines hi and wait
I2C_SDA_LO;  					I2CDelay(I2C_MINTIME);	// data line low and wait
I2C_ALL_LO;												// also clock line low
}

void I2CSendStop(void)  REWRITABLE;
void I2CSendStop(void)
{
I2C_ALL_LO;						I2CDelay(I2C_MINTIME);	// all lines low and wait
I2C_SDA_LO;						I2CDelay(I2C_MINTIME);	// clock hi and wait
I2C_ALL_HI;												// also data line hi
}

void I2CSend0(void)  REWRITABLE;
void I2CSend0(void)
{
I2C_ALL_LO;						I2CDelay(I2C_MINTIME); 	//apply data=0 and wait
I2C_SDA_LO;						I2CDelay(I2C_MINTIME);	//clock hi and wait 
I2C_ALL_LO;												//clock low														
}

void I2CSend1(void)  REWRITABLE;
void I2CSend1(void)
{
I2C_SCL_LO;						I2CDelay(I2C_MINTIME);	//apply data hi and wait
I2C_ALL_HI;						I2CDelay(I2C_MINTIME);	//also clock hi and wait
I2C_SCL_LO;												//clock low
}

char I2CGetBit(void)  REWRITABLE;
char I2CGetBit(void)									//open collector
{
char b;
I2C_SCL_LO;						I2CDelay(I2C_MINTIME);	//SDA Hi Z and wait
I2C_ALL_HI;						I2CDelay(I2C_MINTIME);	//clock hi and wait
b = (I2C_PIN & SDA) >> BIT_SDA;							//get bit
I2C_SCL_LO;												//clock low
return b;
}

void I2CSendByte(char b)  REWRITABLE;
void I2CSendByte(char b)
{
char i,p;
	p = 0x80;
    for (i=0; i<8; i++){
    	if ((p & b) == 0) I2CSend0(); else I2CSend1();
    	p = p >> 1;
  		};
    I2CErrors = I2CErrors + I2CGetBit(); 			//Acknowledge
  	return; 
}

char I2CReceiveByte(void)  REWRITABLE;
char I2CReceiveByte(void)							//open collector
{
char i;
char b = 0;
    for (i=0; i<8; i++){
		b = b << 1;
    	b = b | I2CGetBit();
  		};
	I2CSend0();										//send Acknowledge
  	return b;
}

char I2CReceiveLastByte(void)  REWRITABLE;
char I2CReceiveLastByte(void)						//open collector
{
char i;
char b = 0;
    for (i=0; i<8; i++){
		b = b << 1;
    	b = b | I2CGetBit();
  		};
	I2CSend1();										//send Acknowledge
  	return b;
}


char I2CGetByte(char adr) REWRITABLE;
char I2CGetByte(char adr){
I2CSendStart();
I2CSendByte(adr | 1);								// send device address from "value"
char result=I2CReceiveLastByte();					// receive data and return
I2CSendStop();
return result;
}

