//////////////////////////////////////////////////////////////////////////////////////////////
//	LCD library for 4 bit communication with a standard alphanumerical LCD, optionally via I2C
//  supported LCD controller: NT3881 and compatible ones
//  Created May 5th, 2009 by Thomas Baier DG8SAQ, all rights reserved
//  Revision date 5.5.2009
//////////////////////////////////////////////////////////////////////////////////////////////


////////user settings
#define PCF8574 0x40			//I2C addresses
#define PCF8574A 0x70
// don't change this block
#define P_I2C 0
#define P_A 1
#define P_B 2
#define P_C 3
#define P_D 4
// end don't change
// LCD D0...D2 connected to GND, D3 connected to VCC
#define LCD__PORT P_I2C		//assumes, that LCD D4...D7 are connected to LCD_PORT0...3
#define LCD_E (1<<4)
#define LCD_RS (1<<5)
#define LCD_I2C PCF8574A
#define I2C_DDR DDRB		
#define I2C_PIN PINB
#define I2C_PORT PORTB
////////end user settings
								
#if LCD__PORT == P_A
#define LCD_PORT PORTA 
#define LCD_PIN  PINA
#define LCD_DDR  DDRA
#endif 
#if LCD__PORT == P_B
#define LCD_PORT PORTB 
#define LCD_PIN  PINB
#define LCD_DDR  DDRB
#endif
#if LCD__PORT == P_C
#define LCD_PORT PORTC 
#define LCD_PIN  PINC
#define LCD_DDR  DDRC
#endif
#if LCD__PORT == P_D
#define LCD_PORT PORTD 
#define LCD_PIN  PIND
#define LCD_DDR  DDRD
#endif

#define LCD_CLEAR 		1
#define LCD_CHOME   	2
#define LCD_ENTRY_MODE	4
#define LCD_COFF    	0x0c
#define LCD_CBLINK  	0x0f
#define LCD_CON      	0x0e
#define LCD_DDRAM   	0x80	//addresses: line 1: 0-3Fh, line 2:40h-7Fh

#if LCD__PORT == P_I2C	//I2C
#define LCD_SHORT_DELAY 10	
#define LCD_LONG_DELAY 1
#define LCD_outbyte i2c_LCD_outbyte(lcd | 0xC0)   //mask out 2 high bytes as inputs
#else				//port
#define LCD_SHORT_DELAY 40		//us, min 40us
#define LCD_LONG_DELAY 2		//ms, min 2ms
#define LCD_outbyte LCD_PORT=lcd
#endif

#define LCD_shortdelay() _delay_us(LCD_SHORT_DELAY)
#define LCD_longdelay() _delay_ms(LCD_LONG_DELAY)

#define LCD_DISPLAY_ON 		4
#define LCD_CURSOR_ON 		2
#define LCD_CURSOR_BLINK 	1

USERDATA 		  unsigned char 		lcd;	//LCD interface byte

#if LCD__PORT == P_I2C	//I2C
void i2c_LCD_outbyte(unsigned char i) REWRITABLE;
void i2c_LCD_outbyte(unsigned char i){
I2CSendStart();					// send I2C start sequence 
I2CErrors = 0;					// reset error counter
I2CSendByte(LCD_I2C);
I2CSendByte(i);
I2CSendStop();
}
#endif
						
void LCD_Strobe(void) REWRITABLE;
void LCD_Strobe(void){
lcd=lcd & ~LCD_E;
LCD_outbyte;
_delay_us(1);
lcd=lcd | LCD_E;
LCD_outbyte;
}				

void LCD_SendByte(unsigned char i) REWRITABLE;
void LCD_SendByte(unsigned char i){
unsigned char c;
c=(i & 0xF0) >>4;
lcd=lcd & 0xF0;
lcd=lcd | c;
LCD_outbyte;		//out high nibble
LCD_Strobe();
LCD_shortdelay();
c=i & 0xF;
lcd=lcd & 0xF0;
lcd=lcd | c;
LCD_outbyte;
LCD_Strobe();
LCD_shortdelay();
}

void LCD_SendNibble(unsigned char i) REWRITABLE;
void LCD_SendNibble(unsigned char i){
i=i & 0xF;
lcd=lcd & 0xF0;
lcd=lcd | i;
LCD_outbyte;		//out high nibble
LCD_Strobe();
LCD_shortdelay();
}

void LCD_InitPort(void) REWRITABLE;
void LCD_InitPort(void){
#if LCD__PORT != 0
LCD_DDR=LCD_DDR | 0xF | LCD_E | LCD_RS;
lcd=(LCD_PORT & 0xF0 & ~LCD_RS) | LCD_E;
#else
lcd=LCD_E;	
LCD_outbyte;
#endif
}

void LCD_SetCommand(void) REWRITABLE;
void LCD_SetCommand(void){
lcd=lcd & ~LCD_RS;
LCD_outbyte;
}

void LCD_SetData(void) REWRITABLE;
void LCD_SetData(void){
lcd=lcd | LCD_RS;
LCD_outbyte;
}

void LCD_DisplayClear(void) REWRITABLE;
void LCD_DisplayClear(void){
LCD_SendByte(0x01);
LCD_longdelay();					// wait 2ms
}

/*
void LCD_CursorHome(void) REWRITABLE;
void LCD_CursorHome(void){
LCD_SendByte(0x02);
LCD_longdelay();					// wait 2ms
}
*/

void LCD_Cur(char Position)  REWRITABLE;  //Position=Pos+(40h*(Lin-1))
void LCD_Cur(char Position){
LCD_SendByte(Position | LCD_DDRAM);
}

void LCD_HexByte(char b, char pos) REWRITABLE; //write b in hex to LCD Pos
void LCD_HexByte(char b, char pos){
LCD_SetCommand();
LCD_Cur(pos);
LCD_SetData();
char c=(b >> 4);
if (c>9) c=c+7; 
LCD_SendByte(c+'0');
c=(b & 0xf);
if (c>9) c=c+7; 
LCD_SendByte(c+'0');
}

void LCD_SetDisplay(char displaymode) REWRITABLE;
void LCD_SetDisplay(char displaymode){
LCD_SendByte(8|(displaymode & 7));  // Init Display - Command: Display on/off
LCD_shortdelay();					// wait 40us 
}	

void LCD_Init(char displaymode) REWRITABLE;
void LCD_Init(char displaymode){
LCD_SetCommand();
LCD_SendNibble( 0x3);				// set 8 bit mode	
LCD_shortdelay();                   // wait 40us
LCD_SendNibble( 0x3);				// set 8 bit mode again to be sure, for well defined entry
LCD_shortdelay();					// wait 40us 
LCD_SendNibble( 0x2);				// set 4 bit mode
LCD_shortdelay();					// wait 40us 
LCD_SetDisplay(displaymode);		// Init Display - Command: Display on/off
LCD_DisplayClear();					// Display Clear					
LCD_SetData();
}

void LCD_OutNumber(unsigned char bcd[1], unsigned char len, unsigned char dec_place, unsigned char unit) REWRITABLE;
void LCD_OutNumber(unsigned char bcd[1], unsigned char len, unsigned char dec_place, unsigned char unit){
unsigned char i;
char b;
char dp=0;
char leadingzero=1;
if (dec_place==1) leadingzero=0;
len=len>>1;
for (i=0;i<len;i++)
  {
	if (dp==dec_place) LCD_SendByte('.');
	dp++;
	b=bcd[i]>>4;
	if (b>0) leadingzero=0;
	b=b+0x30;
	if (leadingzero==1) b=' ';
	LCD_SendByte(b);
	if (dp==dec_place) LCD_SendByte('.');
	dp++;
	b=bcd[i] & 0xF;
	if (b>0) leadingzero=0;
	b=b+0x30;
	if (leadingzero==1) b=' ';
	LCD_SendByte(b);
  }
if (dp==dec_place) LCD_SendByte('.');
LCD_SendByte(' ');
if (unit>0) LCD_SendByte(unit);
}

void LCD_OutStrP(const char *data, char pos) REWRITABLE;
void LCD_OutStrP(const char *data, char pos){
LCD_SetCommand();
LCD_Cur(pos);
LCD_SetData();
while (pgm_read_byte(data) != 0x00) LCD_SendByte(pgm_read_byte(data++));
}


void LCD_OutChar(char c, char pos) REWRITABLE;
void LCD_OutChar(char c, char pos){
LCD_SetCommand();
LCD_Cur(pos);
LCD_SetData();
LCD_SendByte(c);
}

/*
void LCD_Test(void) REWRITABLE;
void LCD_Test(void){
LCD_InitPort();
LCD_Init(LCD_DISPLAY_ON | LCD_CURSOR_ON | LCD_CURSOR_BLINK);
LCD_SendByte('H');
LCD_SendByte('e');
LCD_SendByte('l');
LCD_SendByte('l');
LCD_SendByte('o');
LCD_SendByte(' ');
LCD_SendByte('W');
LCD_SendByte('o');
LCD_SendByte('r');
LCD_SendByte('l');
LCD_SendByte('d');
LCD_SendByte('!');
}


void LCD_OutFreq(tq f) REWRITABLE;
void LCD_OutFreq(tq f){
LCD_InitPort();
LCD_Init(LCD_DISPLAY_ON | LCD_CURSOR_ON | LCD_CURSOR_BLINK);
f2bcd(f);
LCD_OutNumber(bcd, 10, 4, 'M');
LCD_SendByte('H');
LCD_SendByte('z');
}
*/
