/*---------------------------FunctionSetup(data)--------------------------------*/
/*	Called by USB interface code, via usbFunctionSetup() in main.c, when a      */
/*  control message has been received.                                          */
/*    Returns: 0xFF if 'user supplied function'- in our case FunctionWrite()    */
/*      - should be called.                                                     */
/*      Otherwise, number of bytes to return over USB link. Corresponding data  */
/*      is put into a buffer pointed to by usbMsgPtr                            */
/*                                                                              */
/*  (See MAIN.C and USBDRV.H for details)                                       */
/*  Note that most command processing is done here BUT commands which need      */
/*  access to the input data buffer need to be implemented in FunctionWrite     */
/*------------------------------------------------------------------------------*/

uchar FunctionSetup(uchar data[8]) REWRITABLE;
uchar FunctionSetup(uchar data[8])
{
    static uchar    replyBuf[6];
	
    usbRequest_t *rq = (void *)data;
    usbMsgPtr = replyBuf;	
if(rq->bRequest == 0x10){ 			// EEPROM write byte value=address, index=data
	    eeprom_write_byte((uint8_t *)(data[2]+256*data[3]), data[4]);  
        return 0;
    }

	if(rq->bRequest == 0x11){       	// EEPROM read byte "value"=address
        replyBuf[0] = 
		   eeprom_read_byte((uint8_t *)(data[2]+256*data[3]));
        return 1;
    }	
	uchar i;
					//conserve flash if USB not needed
	if(rq->bRequest == 1){       		// set port B directions
        DDRB = data[2] & 
		 ~((1 << USB_CFG_DMINUS_BIT) 
		 | (1 << USB_CFG_DPLUS_BIT));   // protect USB interface
        return 0;
    }
	if(rq->bRequest == 2){       		// read port B 
        replyBuf[0] = PINB;
        return 1;
	}
	if(rq->bRequest == 3){       		// read port B states 
        replyBuf[0] = PORTB;
        return 1;
    }
	if(rq->bRequest == 4){       		// set port B outputs 
        PORTB = data[2] & 
		 ~((1 << USB_CFG_DMINUS_BIT) 
		 | (1 << USB_CFG_DPLUS_BIT));   // protect USB interface
        return 0;
    }
	if(rq->bRequest == 5){       		// send I2C start sequence 
        I2CSendStart();
        return 0;
    }	
	if(rq->bRequest == 6){       		// send I2C stop sequence
        I2CSendStop();
        return 0;
    }	
	if(rq->bRequest == 7){       		// send byte to I2C 
		I2CErrors = 0;					// reset error counter
        I2CSendByte(data[2]);
		replyBuf[0] = I2CErrors;		// return number of I2C transmission errors	
        return 1;
    }	
	if(rq->bRequest == 8){       		// send word to I2C 
		I2CErrors = 0;					// reset error counter
        I2CSendByte(data[2]);
        I2CSendByte(data[3]);
		replyBuf[0] = I2CErrors;		// return number of I2C transmission errors	
        return 1;
    }	
	if(rq->bRequest == 9){       		// send dword to I2C 
		I2CErrors = 0;					// reset error counter
        I2CSendByte(data[2]);
        I2CSendByte(data[3]);
        I2CSendByte(data[4]);
        I2CSendByte(data[5]);
		replyBuf[0] = I2CErrors;		// return number of I2C transmission errors	
        return 1;
    }
	if(rq->bRequest == 0xa){       		// send word to I2C with start and stop sequence
        I2CSendStart();					// send start sequence
        I2CSendByte(data[2] & 0xFE);  	// send device address from "value"
        I2CSendByte(data[4]);			// send data from "index"
        I2CSendStop();					// send stop sequence
		replyBuf[0] = I2CErrors;		// return number of I2C transmission errors									
        return 1;
    }	
	if(rq->bRequest == 0xb){       		// receive word from I2C with start and stop sequence
        I2CSendStart();
        I2CSendByte(data[2] | 1);		// send device address from "value"
        replyBuf[1] = I2CReceiveLastByte();	// receive data and return
        I2CSendStop();
		replyBuf[0] = I2CErrors;		// return number of I2C transmission errors	
        return 2;
	}
#ifdef DG8SAQ_I2C
	if(rq->bRequest == 0xc){       		// modify I2C clock
        I2C_MINTIME = data[2]+256*data[3];// set time constant from "value"
        return 0;
    }
#endif
	if(rq->bRequest == 0xd){       		// read OSCCAL to "value"
        replyBuf[0] = OSCCAL;
        return 1;
    }
	if(rq->bRequest == 0xe){       		// Write "value" to OSCCAL
        OSCCAL = data[2];
        return 0;
    }
/////SI570 specific code/////////////////////////////////////////////////////
	if(rq->bRequest == 0x20){       	// SI570: write byte from register index
        I2CSendStart();					
        I2CSendByte((data[2]<<1)&0xFE);	// send device address from "value" low
		I2CSendByte(data[3]);			// send Byte address from "value" high
		I2CSendByte(data[4]);	    	// send data from "index" low
        I2CSendStop();
		replyBuf[0] = I2CErrors;		// return number of I2C transmission errors	
        return 1;
	}
	if(rq->bRequest == 0x21){       	// SI570: read byte to register index
        I2CSendStart();					
        I2CSendByte((data[2]<<1)&0xFE); // send device address from "value" low
		I2CSendByte(data[3]);			// send Byte address from "value" high
		I2CSendStart();
		I2CSendByte((data[2]<<1)|1);	// send device address from "value" low, ready for read
        replyBuf[1] = I2CReceiveLastByte();	// receive data and return
        I2CSendStop();
		replyBuf[0] = I2CErrors;		// return number of I2C transmission errors	
        return 2;
	}
	if(rq->bRequest == 0x22){       	// SI570: freeze NCO 
		I2C_adr = data[2];
		SI570_freezeNCO();
		replyBuf[0] = I2CErrors;		// return number of I2C transmission errors	
        return 1;
	}
	if(rq->bRequest == 0x23){       	// SI570: unfreeze NCO
		I2C_adr = data[2];
		SI570_unfreezeNCO();
		replyBuf[0] = I2CErrors;		// return number of I2C transmission errors	
        return 1;
	}
	if((rq->bRequest >= 0x30) &&
	  (rq->bRequest <=  0x38)){       	// use usbFunctionWrite to transfer data
		I2C_adr = data[2];	
		byte_adr= data[3];
		command=rq->bRequest;			
        return 0xff;
	}
	
/*	if(rq->bRequest == 0x3e){       	// debug: read out calculated frequency control registers
		for (i=0;i<6;i++){
			replyBuf[i] = SI570_data[i];// copy data bytes
			}
        return 6;}*/
	if(rq->bRequest == 0x3f){       	// SI570: read out frequency control registers
        I2CSendStart();					// read all registers in one block
        I2CSendByte(data[2]<<1);	    // send device address from "value" lo
		I2CSendByte(7);			    	// send Byte address 7 of first register
		I2CSendStart();	
		I2CSendByte((data[2]<<1)|1);	// send device address from "value" lo for reading
		for (i=0;i<5;i++){
			replyBuf[i] = I2CReceiveByte();	// receive data byte
			}
		replyBuf[5] = I2CReceiveLastByte(); // receive without Acknowledge
        I2CSendStop(); 
	/*	for (i=0;i<6;i++){				// alternatively, read register after register
      	I2CSendStart();					
        I2CSendByte((data[2]<<1)&0xFE); // send device address from "value" low
		I2CSendByte(7+i);				// send Byte address from "value" high
		I2CSendStart();
		I2CSendByte((data[2]<<1)|1);	// send device address from "value" low, ready for read
        replyBuf[i] = I2CReceiveLastByte();	// receive data without Acknowledge and return
        I2CSendStop();
		}*/
        return 6;
	}
	if(rq->bRequest == 0x40){       	// return number of I2C transmission errors
		replyBuf[0] = I2CErrors;		
        return 1;
	}

	if(rq->bRequest == 0x50){       	// set RXTX and get cw-key status
	    if (data[2] == 0) PORTB = PORTB & ~RXTX;
		else PORTB = PORTB | RXTX;
		replyBuf[0] = PINB & CWKEY;     // read SDA and cw key level simultaneously
        return 1;
    }
	if (rq->bRequest == 0x53){		// set oscillator frequency multiplier/divider
		if (data[2] < MIN_FREQ_MULT || data[2] > MAX_FREQ_MULT ||
			data[3] < MIN_FREQ_DIV || data[3] > MAX_FREQ_DIV) {	// Rather arbitrary check on multiplier/divider
			replyBuf[0] = 0xFE;		// 0xFE => parameter not supported
			return 1;
		}
	
		Si570_freq_mult = data[2];
		Si570_freq_div = data[3];
		return 0;
	}
	if (rq->bRequest == 0x54){		// Read oscillator frequency multiplier/divider
		replyBuf[0] = Si570_freq_mult;
		replyBuf[1] = Si570_freq_div;
		return 2;
	}
/*
/////////////////////////////////debug LCD commands///
	if(rq->bRequest == 0x80){ 
		LCD_SetCommand();
    	return 0;
    } 
	if(rq->bRequest == 0x81){ 
		LCD_SetData();
    	return 0;
    }
	if(rq->bRequest == 0x82){ 
		LCD_SendByte(data[2]);
    	return 0;
    }
	if(rq->bRequest == 0x83){ 
		LCD_InitPort();
		LCD_Init(data[2]);
    	return 0;
    }
	if(rq->bRequest == 0x85){       	// send byte to LCD 
		I2CSendStart();					// send I2C start sequence 
		I2CErrors = 0;					// reset error counter
        I2CSendByte(LCD_I2C);
        I2CSendByte(data[2]);
		I2CSendStop();					// send I2C stop sequence
		replyBuf[0] = I2CErrors;		// return number of I2C transmission errors	
        return 1;
    }
	if(rq->bRequest == 0x86){ 
		LCD_InitPort();
		LCD_Init(LCD_DISPLAY_ON | LCD_CURSOR_ON | LCD_CURSOR_BLINK);
		f2bcd((tq)freq.qw);
		LCD_OutNumber(bcd, 10, 4, 'M');
		LCD_SendByte('H');
		LCD_SendByte('z');
		replyBuf[0]=0x86;
    	return 1;
    }
	if(rq->bRequest == 0x87){ 
	    DDRB=DDRB & 0xF;
	    lcd=0xc0;
		LCD_Test();
		LCD_Init(LCD_DISPLAY_ON );
		PCMSK |= (1<<PCINT5); //  tell pin change mask to listen to pin1/PCINT5
   		GIMSK |= (1<<PCIE); // enable PCINT interrupt in the general interrupt mask
		Timer0_On();
    	return 0;
    }
	if(rq->bRequest == 0x88){ 
		LCD_Cur(data[2]);
	return 0;
	}
	if(rq->bRequest == 0x89){ 
	    LCD_HexByte(data[2],data[4]);
		return 0;
	}
	if(rq->bRequest == 0x8a){ 
	    LCD_HexByte(PINB,0x40);
		return 0;
	}
	if(rq->bRequest == 0x8f){ 
		LCD_OutFreq((tq)freq.qw);
		replyBuf[0]=0x8f;
    	return 1;
    }


	if(rq->bRequest == 0x8F){       		// set portB
	    //f2bcd((tq)freq.qw);
		f2bcd((tq)(unsigned long)(1589076522));
		LCD_InitPort();
		LCD_Init(LCD_DISPLAY_ON | LCD_CURSOR_ON | LCD_CURSOR_BLINK);
		LCD_OutNumber(bcd, 10, 4, 'k');
		//LCD_SendByte('H');
		//LCD_SendByte('z');
		replyBuf[0]=0x8F;
        return 1;
    }*/
////////////////////////////////////
	if (rq->bRequest == 0xFE){		// timers and pcints off
		Timer0_Off();
		GIMSK &= ~(1<<PCIE); 	// enable PCINT interrupt in the general interrupt mask		
		return 1;
	}


	if (rq->bRequest == 0xFF){		// Return firmware version number
		replyBuf[0] = (MAJ_VER << 4) | MIN_VER;
		return 1;
	}
	replyBuf[0] = 0xff;					// return value 0xff => command not supported 
    return 1;
}


/*--------------------------FunctionWrite(data, len)----------------------------*/
/*	Called by USB interface code, via usbFunctionWrite() in main.c, when a      */
/*  packet has been received and FunctionSetup returns 0xFF                     */
/*    Returns: 0xFF if an error has occurred                                    */
/*             0 if more data is expected                                       */
/*             non-zero if all data received and processed                      */
/*                                                                              */
/*  (See MAIN.C and USBDRV.H for details)                                       */
/*------------------------------------------------------------------------------*/

uchar FunctionWrite(uchar *data, uchar len)  REWRITABLE;
uchar FunctionWrite(uchar *data, uchar len) 
{
uchar i;
	if (command==0x30)					// get register contents and load to SI570
		{
		if (len<6) return 1;
		for (i=0;i<6;i++) SI570_data[i]=data[i];
		SI570_Load();
		return 1;
		}

	if (len<4) return 1;

	if (command==0x32)					// get frequency and load to SI570
		{	
		for (i=0;i<4;i++) freq.bytes[i]=data[i];
		byte_adr = 7;
		if (SI570CalcRegs()>0) SI570_Load();
		return 1;
		}
	if (command==0x33)					// write new crystal frequency to EEPROM
		{	
		for (i=0;i<4;i++) {
			eeprom_write_byte((uint8_t *)(F_CRYST+i), data[i]);
			fcr.bytes[i]=data[i];}
		eeprom_write_byte((uint8_t *)(F_CRYST_STATUS), 0);
		return 1;
		}
	if (command==0x34)					// get frequency and load to SI570
		{	
		for (i=0;i<4;i++) freq.bytes[i]=data[i];
		SetFrequency();
		return 1;
		} 
	if (command==0x38)
		{
		for (i=0;i<4;i++) freq.bytes[i]=data[i];
		SetFreqAndDisplay();//?????????
		}
	return 1;
}
