/* Name: timer.h
 * Project: SI570 controller based on ObDev's PowerSwitch based on AVR USB driver
 * Author: Thomas Baier DG8SAQ (SI570 extensions)
 * Creation Date: 2007-07-14
 * Copyright: (c) 2007...2009 by DG8SAQ
 * License: GNU GPL v2 (see License.txt) or proprietary (CommercialLicense.txt)
 * This Revision: 
 */


//Timer0 Settings
#define TIMER0_OFF 		0
#define TIMER0_DIV1		1 
#define TIMER0_DIV8		2 
#define TIMER0_DIV64	3 
#define TIMER0_DIV256	4 
#define TIMER0_DIV1024	5
#define TIMER0_T0_FALL	6 
#define TIMER0_T0_RISE	7


extern unsigned long frequency;
extern freq_t freq;
extern char SetFreqAndDisplay(void);
extern volatile char fmult, fsep;
extern uint8_t unit;
extern char units[8];
extern signed char Si570_freq_mult;
extern signed char Si570_freq_div;
extern long Si570_freq_offset;
extern signed char USB_on;						// if 0 USB support switched off, else 0xFF;
extern signed char MemorySpace;					// for save/recall frequency
extern signed char OffsetSign;					// selected sign of frequency offset (1:+,0:-)
extern unsigned char I2C_adr;
extern signed char Si570_freq_mult;
extern signed char Si570_freq_div;
extern long Si570_freq_offset;
extern unsigned long Si570_centre;
//extern uint16_t jump_table[];


#define MENU_TUNE   0	//tune frequency
#define MENU_MENU   1	//select menu
#define MENU_DIAL   2	//dial within selected menu
#define M_SAVE 0
#define M_RETR 1
#define M_MULT 2
#define M_DIV  3
#define M_OFS  4
#define M_I2C  5
#define M_RES  6
#define M_RET  7
#define M_UNIT 0xFF

#ifdef LCD_ON
	char SaveS[]   STRINGS = "Store  ";
	char RecalS[]  STRINGS = "Recall ";
	char MultS[]   STRINGS = "Mult.= ";
	char DivS[]    STRINGS = "Div. = ";
	char OffsS[]   STRINGS = "> Offs.";
	char I2C_S[]   STRINGS = "I2C Ad.";
	char ResS[]    STRINGS = "Reset  ";
	char RetS[]    STRINGS = "       ";
	USERDATA volatile signed char menupos;	//position of cursor
	USERDATA volatile unsigned char select_menu; //selected menu type, can be one of the following
	USERDATA volatile char button, oldbutton;//pushbutton state
	USERDATA volatile unsigned char digit;   //frequency digit to be tuned
	USERDATA volatile signed char menu;		 //can be one of the following...
	USERDATA volatile unsigned char return_to_freq; //if 1, MENU_TUNE will follow MENU_MENU else MENU_DIAL
#endif

//timer0 control routines/////////////////////////////////////////////////

ISR(TIMER0_OVF_vect){
pt2Timer0_Overflow();
}

void Timer0_On(void) REWRITABLE;
void Timer0_On(void){
TCNT0=0;					//init timer counter
TCCR0A=0;
TCCR0B=TIMER0_DIV1024;		//internal clock prescaler 1024
TIMSK=TIMSK | (1<<TOIE0); 	//enable overflow interrupt 
}

void Timer0_Off(void) REWRITABLE;
void Timer0_Off(void){
TIMSK=TIMSK & ~(1<<TOIE0);	//disable overflow interrupt
TCCR0B=TIMER0_OFF;		    //clock disabled
}

//////////////////////////////////////////////////////////////////////
// store/recall routines

char CheckStateChange(void) REWRITABLE;		//check if multiplier, divider, offset or frequency have changed
char CheckStateChange(void){
unsigned char i; 
freq_t x, y;
i= eeprom_read_byte((uint8_t *)(DEFD)); // check if valid info in EEPROM
if (i!=0) return 1;
i= eeprom_read_byte((uint8_t *)(MUL));
if (i!=Si570_freq_mult) return 1;
i= eeprom_read_byte((uint8_t *)(DIV));
if (i!=Si570_freq_div) return 1;
for (i=0;i<4;i++){
	 x.bytes[i] = eeprom_read_byte((uint8_t *)(i+OFS)); //Read IF offset from EEPROM (4 bytes)
	 y.bytes[i] = eeprom_read_byte((uint8_t *)(i+FREQ));} 
if (x.qw!=Si570_freq_offset) return 1;
if (y.qw!=frequency) return 1;
return 0;
}

void RecallFrequency(char n) REWRITABLE;		//recall device state to EEPROM location n (0...15)
void RecallFrequency(char n){
n=n<<4;
unsigned char i = eeprom_read_byte((uint8_t *)(n+DEFD)); // check if valid info in EEPROM
if (i==0)
  {
   Si570_freq_mult = eeprom_read_byte((uint8_t *)(n+MUL));
   Si570_freq_div  = eeprom_read_byte((uint8_t *)(n+DIV));
   for (i=0;i<4;i++)
        {
		(*(ofs_t *)&Si570_freq_offset).bytes[i] = eeprom_read_byte((uint8_t *)(i+n+OFS)); //Read IF offset from EEPROM (4 bytes)
  // for (i=0;i<4;i++) 
       (*(freq_t *)&frequency).bytes[i] = eeprom_read_byte((uint8_t *)(i+n+FREQ)); }


  }
}

void StoreFrequency(char n) REWRITABLE;		//store device state to EEPROM location n (0...15)
void StoreFrequency(char n){
unsigned char i;
n=n<<4;
eeprom_write_byte((uint8_t *)(n+DEFD),0);   // check if valid info in EEPROM
eeprom_write_byte((uint8_t *)(n+MUL),Si570_freq_mult);
eeprom_write_byte((uint8_t *)(n+DIV),Si570_freq_div);
for (i=0;i<4;i++)
  eeprom_write_byte((uint8_t *)(i+n+OFS),((ofs_t)Si570_freq_offset).bytes[i]); //write IF offset from EEPROM (4 bytes)
for (i=0;i<4;i++)
  eeprom_write_byte((uint8_t *)(i+n+FREQ),((freq_t)frequency).bytes[i]);   	 //write frequency from EEPROM (4 bytes)
}


#ifdef LCD_ON
	//////////////////////////////////////////////////////////////////////
	// recalculate SI570 registers and write to SI570 and display frequency on LCD
	void UpdateDeviceFrequency(void) REWRITABLE;
	void UpdateDeviceFrequency(void){
	wdt_reset();
	freq.qw=frequency;
	SetFreqAndDisplay();
	}
	//////////////////////////////////////////////////////////////////////
	// set type of menu and update display depending on user selection (menupos) after wheel turn
	void MenuSet(void) REWRITABLE;
	void MenuSet(void){
	char c;
	return_to_freq=0;
	if (menupos<15) menupos=menupos+wheel-oldwheel;
	if (menupos<0)  menupos=0;
	if (menupos>=15) 
	  {
	   LCD_OutStrP(RetS,0x47);  //clear trailing displayed data from previous command
	   select_menu=(select_menu+wheel-oldwheel) & 7;
	   menupos=15;
	   if (eeprom_read_byte((uint8_t *)((MemorySpace<<4)+DEFD))==0) c='*'; else c=' ';
	   if( select_menu==M_SAVE ) 
		{
		 LCD_OutStrP(SaveS,0x40);
		 LCD_HexByte(char2bcd(MemorySpace),0x47);
		 LCD_SendByte(' ');
	     LCD_SendByte(c);
		}
	   if( select_menu==M_RETR )
		{
		 LCD_OutStrP(RecalS,0x40);
		 LCD_HexByte(char2bcd(MemorySpace),0x47);
		 LCD_SendByte(' ');
	     LCD_SendByte(c);
		}
	   if( select_menu==M_MULT )
		{
		 LCD_OutStrP(MultS,0x40);
		 LCD_HexByte(char2bcd(Si570_freq_mult),0x47);
		}
	   if( select_menu==M_DIV )
		{
		 LCD_OutStrP(DivS,0x40);
		 LCD_HexByte(char2bcd(Si570_freq_div),0x47);
		}
	   if( select_menu==M_OFS )
		{
		 LCD_OutStrP(OffsS,0x40);
		 if (OffsetSign==1) LCD_OutChar('+',0x41); else LCD_OutChar('-',0x41);
		}
	   if( select_menu==M_I2C )
		{
		 LCD_OutStrP(I2C_S,0x40);
		 LCD_HexByte(I2C_adr,0x48);
		 LCD_SendByte('H');
//		 LCD_SendByte('e');
//		 LCD_SendByte('x');
		}
	   if( select_menu==M_RES )
		{
		 return_to_freq=1;
		 LCD_OutStrP(ResS,0x40);
		}
	   if( select_menu==M_RET )
		{
		 return_to_freq=1;
		 LCD_OutStrP(RetS,0x40);
		}
	  }
	if (menupos<=10)
	  {
	   return_to_freq=1;
	   digit=10-menupos;
	   if(menupos<fsep) digit--;
	   }
	if (menupos == 12) 
	  {
	   select_menu=M_UNIT; //displayed unit
	  }
	LCD_SetCommand();
	LCD_Cur(menupos);
	LCD_SetDisplay(LCD_DISPLAY_ON | LCD_CURSOR_ON);
	}


	///////////////////////////////////////////////////////////////////////////////////
	// Do menu dial action here
	void MenuDial(void) REWRITABLE;
	void MenuDial(void){
	char c;
	if(( select_menu==M_SAVE ) | ( select_menu==M_RETR ))
	  {
	   MemorySpace=MemorySpace+wheel-oldwheel;
	   if (MemorySpace>15) MemorySpace=15;
	   if (MemorySpace<1) MemorySpace=1;
	   LCD_HexByte(char2bcd(MemorySpace),0x47);
	   if (eeprom_read_byte((uint8_t *)((MemorySpace<<4)+DEFD))==0) c='*'; else c=' ';
	   LCD_SendByte(' ');
	   LCD_SendByte(c);
	  }
	if( select_menu==M_MULT )
	  {
	   Si570_freq_mult=Si570_freq_mult+wheel-oldwheel;
	   if (Si570_freq_mult>MAX_FREQ_MULT) Si570_freq_mult=MAX_FREQ_MULT;
	   if (Si570_freq_mult<MIN_FREQ_MULT) Si570_freq_mult=MIN_FREQ_MULT;
	   LCD_HexByte(char2bcd(Si570_freq_mult),0x47);
	  }
	if( select_menu==M_DIV )
	  {
	   Si570_freq_div=Si570_freq_div+wheel-oldwheel;
	   if (Si570_freq_div>MAX_FREQ_DIV) Si570_freq_div=MAX_FREQ_DIV;
	   if (Si570_freq_div<MIN_FREQ_DIV) Si570_freq_div=MIN_FREQ_DIV;
	   LCD_HexByte(char2bcd(Si570_freq_div),0x47);
	  }
	if( select_menu==M_OFS )
	  {
	   OffsetSign=(OffsetSign+wheel-oldwheel)&1;
	   if (OffsetSign==1) LCD_OutChar('+',0x41); else LCD_OutChar('-',0x41);
	  }
	if( select_menu==M_I2C )
	  {
	   I2C_adr=I2C_adr+wheel-oldwheel;
	   LCD_HexByte(I2C_adr,0x48);
	   LCD_SendByte('H');
//	   LCD_SendByte('e');
//	   LCD_SendByte('x');
	   eeprom_write_byte((uint8_t *)I2C_ADR,I2C_adr);
	  }
	if ( select_menu==M_UNIT)
	  {
	   wdt_reset();
	   unit=unit+wheel-oldwheel;
	   unit=unit & 3;
	   uint8_t u=unit<<1;
	   fmult=pgm_read_byte(&units[u]);
	   fsep=pgm_read_byte(&units[u+1]);
	   freq.qw=frequency;
	   SetFreqAndDisplay();
	  }
	}


	//////////////////////////////////////////////////////////////////////
	void MenuDo(void) REWRITABLE;
	void MenuDo(void){
	unsigned int i;
	LCD_OutStrP(RetS,0x40); //clear second LCD line
	LCD_OutStrP(RetS,0x47); //clear second LCD line
	if( select_menu==M_SAVE ) 
	  {
	   StoreFrequency(MemorySpace);
	  }
	if( select_menu==M_RETR )
	  {
	   RecallFrequency(MemorySpace);
	  }
	if( select_menu==M_OFS )
	  {
	   if (OffsetSign==1) Si570_freq_offset=frequency; else Si570_freq_offset=-frequency;
	  }
	if( select_menu==M_RES )
	  {
	   for (i=0x0;i<0x200;i++)
	     {
		  eeprom_write_byte((uint8_t *)(i),0xFF);
		  wdt_reset();
		 }
		  Si570_centre = 0;
		  Si570_freq_mult = DEF_FREQ_MULT;
		  Si570_freq_div = DEF_FREQ_DIV;
		  Si570_freq_offset = 0;
		  frequency=000000;
	  }
	UpdateDeviceFrequency();
	}
#endif

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void Timer0_Overflow(void)  REWRITABLE;
void Timer0_Overflow(void) 
{
#ifdef LCD_ON					
	Timer0_Off();
	sei();
	button=I2CGetByte(LCD_I2C) & 0x40;	//get turnwheel pushbutton input state, low if pressed!
	////////////////////////////////////////////
	/// START action upon event pushbutton press
	if (button==0) //button pressed
	  {
	    if (oldbutton!=0) //change menu state if button down
		  {
		   oldbutton=button;
		   oldwheel=wheel;
			  if (menupos>=15)
			    {
				 digit=0;
				 menupos=10;
				}	   
		   if ((menu==MENU_TUNE) | (menu==MENU_DIAL))
		     {
			  menu=MENU_MENU;
			  MenuSet();
	         }
			 else //previous state MENU select
			 {
			  if (return_to_freq==1) 
			    {
				 menu=MENU_TUNE;
				 MenuDo();
				 select_menu=M_RET;
				} else menu=MENU_DIAL;
			  LCD_SetCommand(); //prepare for setting cursor
			  LCD_SetDisplay(LCD_DISPLAY_ON); //cursor off
			 }
	  	   LCD_SetData(); //cursor setting done    
		  }	
		Timer0_On();
	  	return;
	  } 
	/// END action if pushbutton just got pressed
	  else // button not pressed
	  {
	    if (oldbutton==0) // but just returned from pressed, i.e. button just released
		  {
		   oldbutton=button;
		   oldwheel=wheel;
	       Timer0_On();
	  	   return;        //no action on pushbutton release
		  }	   
	  } 
	if (wheel==oldwheel) 
	  {
	   	Timer0_On();
	  	return;           //no action on no wheel turn
	  } 
	///////////////////////////////////
	///START action on event WHEEL TURN

	if (menu==MENU_TUNE)
	  {
	   frequency=frequency+pgm_read_dword(&Pow10[digit])*(wheel-oldwheel);
	   UpdateDeviceFrequency();
	   if (CheckStateChange()==1) StoreFrequency(0); //autosave settings if changed
	  }
	if (menu==MENU_MENU) MenuSet();
	if (menu==MENU_DIAL) MenuDial();
	oldwheel=0;
	wheel=0;
	Timer0_On();
	return;
#endif
}
